import { z } from "zod";
import { RequestError } from "./error";
import { InternalRequestSchema, InternalResponseSchema, } from "./messages";
export default class Service {
    name;
    adaptors;
    constructor(name) {
        this.name = name;
        this.adaptors = {};
    }
    addAdaptor(name, adaptor) {
        this.adaptors[name] = adaptor;
    }
    subscribe(adaptor, subject, fn, schema = undefined) {
        this.adaptors[adaptor].subscribe(`${this.name}.${subject}`, async (rawReq) => {
            const rawReqJson = JSON.parse(rawReq);
            const internalRequestJson = InternalRequestSchema.safeParse(rawReqJson);
            if (!internalRequestJson.success) {
                return JSON.stringify({
                    statusCode: 400,
                    data: "bad request structure",
                });
            }
            const internalRequest = internalRequestJson.data;
            const req = {
                service: internalRequest.from,
                subject: subject,
            };
            if (internalRequest.data && schema) {
                const validate = schema.safeParse(internalRequest.data);
                if (!validate.success) {
                    return JSON.stringify({
                        statusCode: 400,
                        data: validate.error,
                    });
                }
                req.data = validate.data;
            }
            try {
                const res = await fn(req);
                const internalResponse = {
                    statusCode: res.statusCode,
                    data: res.data,
                };
                return JSON.stringify(internalResponse);
            }
            catch (err) {
                if (err instanceof RequestError) {
                    return JSON.stringify({
                        statusCode: err.statusCode,
                        data: err.message,
                    });
                }
                return JSON.stringify({
                    statusCode: 500,
                    data: "unknow error apend",
                });
            }
        });
    }
    async request(adaptor, req, schema) {
        if (!this.adaptors[adaptor]) {
            throw new Error(`${adaptor} adaptor not exist`);
        }
        const internalRequest = {
            from: this.name,
            data: req.data,
        };
        const internalRequestJson = JSON.stringify(internalRequest);
        try {
            const rawRes = await this.adaptors[adaptor].request(`${req.service}.${req.subject}`, internalRequestJson);
            const rawResJson = JSON.parse(rawRes);
            const internalResponseJson = InternalResponseSchema.safeParse(rawResJson);
            if (!internalResponseJson.success) {
                throw new RequestError(internalResponseJson.error.toString(), 500);
            }
            const internalResponse = internalResponseJson.data;
            if (internalResponse.statusCode < 200 ||
                internalResponse.statusCode >= 299) {
                throw new RequestError(internalResponse.data, internalResponse.statusCode);
            }
            const res = {
                statusCode: internalResponse.statusCode,
            };
            if (internalResponse.data && schema) {
                const validate = schema.safeParse(internalResponse.data);
                if (!validate.success) {
                    throw new RequestError(validate.error.message, 400);
                }
                res.data = validate.data;
            }
            return res;
        }
        catch (err) {
            if (err instanceof RequestError) {
                throw err;
            }
            throw new RequestError("unexpected error", 500);
        }
    }
    async listen() {
        for (const index in this.adaptors) {
            await this.adaptors[index].listen(this.name);
        }
    }
    async stop() {
        for (const index in this.adaptors) {
            await this.adaptors[index].stop();
        }
    }
}
